@@ -33,6 +33,11 @@ from zipfile import (
 )
 
 
+# Constants for compatibility modes
+PY313_COMPAT = sys.version_info < (3, 14)  # disable zstd
+PY36_COMPAT = sys.version_info < (3, 7)  # disable compress_level
+PY35_COMPAT = sys.version_info < (3, 6)  # backport ZipInfo functions, stringify path-like objects
+
 # Size of chunks to read out of files
 # Note that when compressing data the compressor will operate on bigger chunks
 # than this - it keeps a cache as new chunks are fed to it.
@@ -51,16 +56,22 @@ ZIP64_ESTIMATE_FACTOR = 1.05
 # (includes "/" regardless of platform as per ZIP format specification)
 PATH_SEPARATORS = set(x for x in (os.sep, os.altsep, "/") if x)
 
-# Constants for compatibility modes
-PY36_COMPAT = sys.version_info < (3, 7)  # disable compress_level
-PY35_COMPAT = sys.version_info < (3, 6)  # backport ZipInfo functions, stringify path-like objects
+# zstd-related constants
+if not PY313_COMPAT:
+    from zipfile import ZIP_ZSTANDARD, ZSTANDARD_VERSION
+    from compression.zstd import CompressionParameter
+    ZSTD_LEVEL_BOUNDS = CompressionParameter.compression_level.bounds()
 
 
 __all__ = [
     # Defined classes
     "ZipStream", "ZipStreamInfo",
     # Compression constants (imported from zipfile)
-    "ZIP_STORED", "ZIP_DEFLATED", "BZIP2_VERSION", "ZIP_BZIP2", "LZMA_VERSION", "ZIP_LZMA",
+    "ZIP_STORED",
+    "ZIP_DEFLATED",
+    "ZIP_BZIP2", "BZIP2_VERSION",
+    "ZIP_LZMA", "LZMA_VERSION",
+    *(["ZIP_ZSTANDARD", "ZSTANDARD_VERSION"] if not PY313_COMPAT else []),
     # Helper functions
     "walk"
 ]
@@ -83,14 +94,34 @@ def _check_compression(compress_type, compress_level):
         __log__.warning(
             "compress_level has no effect when using ZIP_STORED/ZIP_LZMA"
         )
-    elif compress_type == ZIP_DEFLATED and not 0 <= compress_level <= 9:
-        raise ValueError(
-            "compress_level must be between 0 and 9 when using ZIP_DEFLATED"
-        )
-    elif compress_type == ZIP_BZIP2 and not 1 <= compress_level <= 9:
-        raise ValueError(
-            "compress_level must be between 1 and 9 when using ZIP_BZIP2"
-        )
+    elif compress_type == ZIP_DEFLATED:
+        if not 0 <= compress_level <= 9:
+            raise ValueError(
+                "compress_level must be between 0 and 9 when using ZIP_DEFLATED"
+            )
+    elif compress_type == ZIP_BZIP2:
+        if not 1 <= compress_level <= 9:
+            raise ValueError(
+                "compress_level must be between 1 and 9 when using ZIP_BZIP2"
+            )
+    elif not PY313_COMPAT and compress_type == ZIP_ZSTANDARD:
+        if not ZSTD_LEVEL_BOUNDS[0] <= compress_level <= ZSTD_LEVEL_BOUNDS[1]:
+            raise ValueError(
+                "compress_level must be between {} and {} when using ZIP_ZSTANDARD".format(
+                    *ZSTD_LEVEL_BOUNDS
+                )
+            )
+
+
+def _min_version_for_compress_type(compress_type, min_version=0):
+    """Ensure the compress_type is supported by the min_version"""
+    if compress_type == ZIP_BZIP2:
+        min_version = max(BZIP2_VERSION, min_version)
+    elif compress_type == ZIP_LZMA:
+        min_version = max(LZMA_VERSION, min_version)
+    elif not PY313_COMPAT and compress_type == ZIP_ZSTANDARD:
+        min_version = max(ZSTANDARD_VERSION, min_version)
+    return min_version
 
 
 def _timestamp_to_dos(ts):
@@ -175,11 +206,7 @@ class ZipStreamInfo(ZipInfo):
             file_size = 0xFFFFFFFF
             compress_size = 0xFFFFFFFF
 
-        if self.compress_type == ZIP_BZIP2:
-            min_version = max(BZIP2_VERSION, min_version)
-        elif self.compress_type == ZIP_LZMA:
-            min_version = max(LZMA_VERSION, min_version)
-
+        min_version = _min_version_for_compress_type(self.compress_type, min_version)
         self.extract_version = max(min_version, self.extract_version)
         self.create_version = max(min_version, self.create_version)
         filename, flag_bits = self._encodeFilenameFlags()
@@ -313,11 +340,7 @@ class ZipStreamInfo(ZipInfo):
             ) + extra_data
             min_version = ZIP64_VERSION
 
-        if self.compress_type == ZIP_BZIP2:
-            min_version = max(BZIP2_VERSION, min_version)
-        elif self.compress_type == ZIP_LZMA:
-            min_version = max(LZMA_VERSION, min_version)
-
+        min_version = _min_version_for_compress_type(self.compress_type, min_version)
         extract_version = max(min_version, self.extract_version)
         create_version = max(min_version, self.create_version)
         filename, flag_bits = self._encodeFilenameFlags()
@@ -338,7 +361,7 @@ class ZipStreamInfo(ZipInfo):
             len(filename),
             len(extra_data),
             len(self.comment),
-            0,
+            0,  # disk number this file begins on
             self.internal_attr,
             self.external_attr,
             header_offset
@@ -420,25 +443,20 @@ def _validate_compression(func):
 def _sanitize_arcname(arcname):
     """Terminate the arcname at the first null byte"""
     # based on zipfile._sanitize_filename
-
-    if arcname:
-        # trim the arcname to the first null byte
-        null_byte = arcname.find(chr(0))
-        if null_byte >= 0:
-            arcname = arcname[:null_byte]
-
     if not arcname:
-        raise ValueError(
-            "A valid arcname (name of the entry in the zip file) is required"
-        )
+        return ""
+
+    # trim the arcname to the first null byte
+    null_byte = arcname.find(chr(0))
+    if null_byte >= 0:
+        arcname = arcname[:null_byte]
 
     # Ensure paths in the zip always use forward slashes as the directory
-    # separator
+    # separator and strip any leading ones
     for sep in PATH_SEPARATORS:
         if sep != "/":
             arcname = arcname.replace(sep, "/")
-
-    return arcname
+    return arcname.lstrip("/")
 
 
 def _iter_file(path):
@@ -505,19 +523,24 @@ class ZipStream:
 
         compress_type:
             The ZIP compression method to use when writing the archive, and
-            should be ZIP_STORED, ZIP_DEFLATED, ZIP_BZIP2 or ZIP_LZMA;
-            unrecognized values will cause NotImplementedError to be raised. If
-            ZIP_DEFLATED, ZIP_BZIP2 or ZIP_LZMA is specified but the
-            corresponding module (zlib, bz2 or lzma) is not available,
-            RuntimeError is raised. The default is ZIP_STORED.
+            should be ZIP_STORED, ZIP_DEFLATED, ZIP_BZIP2, ZIP_LZMA, or
+            ZIP_ZSTANDARD (Python 3.14+); unrecognized values will cause
+            NotImplementedError to be raised.
+            If ZIP_DEFLATED, ZIP_BZIP2, ZIP_LZMA, or ZIP_ZSTANDARD is specified
+            but the corresponding module (zlib, bz2, lzma, or compression.zstd)
+            is not available, RuntimeError is raised. The default is ZIP_STORED.
 
         compress_level:
             Controls the compression level to use when writing files to the
-            archive. When using ZIP_STORED or ZIP_LZMA it has no effect. When
-            using ZIP_DEFLATED integers 0 through 9 are accepted (see zlib for
-            more information). When using ZIP_BZIP2 integers 1 through 9 are
-            accepted (see bz2 for more information). Raises a ValueError if the
-            provided value isn't valid for the `compress_type`.
+            archive. When using ZIP_STORED or ZIP_LZMA it has no effect.
+            When using ZIP_DEFLATED integers 0 through 9 are accepted (see zlib
+            for more information).
+            When using ZIP_BZIP2 integers 1 through 9 are accepted (see bz2 for
+            more information).
+            When using ZIP_ZSTANDARD integers -7 though 22 are common (see
+            compression.zstd.CompressionParameter for more information).
+            Raises a ValueError if the provided value isn't valid for the
+            `compress_type`.
 
             Only available in Python 3.7+ (raises a ValueError if used on a
             lower version)
@@ -652,11 +675,14 @@ class ZipStream:
         """Queue up a path to be added to the ZipStream
 
         Queues the `path` up to to be written to the archive, giving it the
-        name provided by `arcname`. If `arcname` is not provided, it is assumed
-        to be the last component of the `path` (Ex: "/path/to/files/" -->
-        "files").
+        name provided by `arcname`.
+        If `arcname` is not provided/empty, it is assumed to be the last
+        component of the `path` (ex: "/path/to/files/" --> "files",
+        "/path/to/file.ext" --> "file.ext").
+        Using an `arcname` of `"/"` is valid when recursing - it causes all
+        the files under `path` to be added at the top level of the zip.
 
-        if `recurse` is `True` (the default), and the `path` is a directory,
+        If `recurse` is `True` (the default), and the `path` is a directory,
         all contents under the `path` will also be added. By default, this is
         done using the `walk` function in this module, which will preserve
         empty directories as well as follow symlinks to files and folders
@@ -690,12 +716,13 @@ class ZipStream:
 
         # special case - discover the arcname from the path
         if not arcname:
-            arcname = os.path.basename(path)
+            arcname = _sanitize_arcname(os.path.basename(path))
             if not arcname:
                 raise ValueError(
                     "No arcname for path '{}' could be assumed".format(path)
                 )
-        arcname = _sanitize_arcname(arcname)
+        else:
+            arcname = _sanitize_arcname(arcname)
 
         # Not recursing - just add the path
         if not recurse or not os.path.isdir(path):
@@ -712,6 +739,9 @@ class ZipStream:
 
         for filepath in recurse(path):
             filename = os.path.relpath(filepath, path)
+            # skip adding the top-level directory if the sanitized arcname is empty
+            if filename == "." and not arcname:
+                continue
             filearcname = os.path.normpath(os.path.join(arcname, filename))
             self._enqueue(
                 path=filepath,
@@ -753,6 +783,8 @@ class ZipStream:
         Raises a RuntimeError if the ZipStream has already been finalized.
         """
         arcname = _sanitize_arcname(arcname)
+        if not arcname:
+            raise ValueError("A valid arcname is required")
 
         if data is None:
             data = b""
@@ -794,6 +826,8 @@ class ZipStream:
     def mkdir(self, arcname):
         """Create a directory inside the ZipStream"""
         arcname = _sanitize_arcname(arcname)
+        if not arcname:
+            raise ValueError("A valid arcname for the directory is required")
 
         if arcname[-1] != "/":
             arcname += "/"
@@ -965,6 +999,8 @@ class ZipStream:
 
     def _enqueue(self, **kwargs):
         """Internal method to enqueue files, data, and iterables to be streamed"""
+        if not kwargs["arcname"].rstrip("/"):
+            raise ValueError("A valid arcname is required")
 
         path = kwargs.get("path")
         data = kwargs.get("data")
@@ -1121,22 +1157,27 @@ class ZipStream:
             centDirOffset > ZIP64_LIMIT or
             centDirSize > ZIP64_LIMIT
         ):
-            # Need to write the Zip64 end-of-archive records
+            # Need to also write a Zip64 end-of-archive record
             zip64EndRec = struct.pack(
                 structEndArchive64,
                 stringEndArchive64,
-                44, 45, 45, 0, 0,
+                44,  # size of this record after this point
+                     # (note: no "zip extensible data" is added so this is a constant)
+                45,  # version made by (Zip64 support)
+                45,  # version needed to extract (Zip64 support)
+                0,  # disk number this record is on
+                0,  # disk number that contains the start of the central directory
                 centDirCount,
                 centDirCount,
                 centDirSize,
-                centDirOffset
+                centDirOffset,
             )
             zip64LocRec = struct.pack(
                 structEndArchive64Locator,
                 stringEndArchive64Locator,
-                0,
+                0,  # disk number where the zip64EndRec starts
                 zip64EndRecStart,
-                1
+                1,  # total number of disks
             )
             yield self._track(zip64EndRec + zip64LocRec)
 
@@ -1147,7 +1188,8 @@ class ZipStream:
         endRec = struct.pack(
             structEndArchive,
             stringEndArchive,
-            0, 0,
+            0,  # disk number this record is on
+            0,  # disk number that contains the start of the central directory
             centDirCount,
             centDirCount,
             centDirSize,
